@@ -1,7 +1,8 @@
 /*
- * panel-menu-bar.c: panel Applications/Actions menu bar
+ * panel-menu-bar.c: panel Applications/Places/Desktop menu bar
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2004 Vincent Untz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +21,7 @@
  *
  * Authors:
  *	Mark McLoughlin <mark@skynet.ie>
+ *	Vincent Untz <vincent@vuntz.net>
  */
 
 #include <config.h>
@@ -32,169 +34,122 @@
 #include "panel-util.h"
 #include "panel-background.h"
 #include "panel-action-button.h"
-#include "panel-stock-icons.h"
-#include "panel-recent.h"
 #include "applet.h"
 #include "menu.h"
+#include "panel-menu-items.h"
 #include "panel-globals.h"
 #include "panel-profile.h"
 #include "panel-lockdown.h"
+#include "panel-stock-icons.h"
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
 struct _PanelMenuBarPrivate {
-	AppletInfo            *info;
-	PanelWidget           *panel;
+	AppletInfo  *info;
+	PanelWidget *panel;
 
-	GtkWidget             *applications_menu;
-	GtkWidget             *actions_menu;
-	GtkWidget             *actions_item;
+	GtkWidget   *applications_menu;
+	GtkWidget   *applications_item;
+	GtkWidget   *places_item;
+	GtkWidget   *desktop_item;
 };
 
 static GObjectClass *parent_class;
-static GtkWidget *panel_menu_bar_create_actions_menu (PanelMenuBar *menubar);
+static GtkWidget *panel_menu_bar_create_applications_menu (PanelMenuBar *menubar);
 
 static void
-panel_menu_bar_show_menu (PanelMenuBar *menubar,
-			  GtkWidget    *menu)
+panel_menu_bar_recreate_menus (PanelMenuBar *menubar)
 {
-	gtk_menu_set_screen (GTK_MENU (menu),
-			     gtk_widget_get_screen (GTK_WIDGET (menubar)));
+	if (menubar->priv->applications_menu) {
+		gtk_widget_destroy (menubar->priv->applications_menu);
+		menubar->priv->applications_menu = panel_menu_bar_create_applications_menu (menubar);
+		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
+					       "menu_panel",
+					       menubar->priv->panel);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->applications_item),
+					   menubar->priv->applications_menu);
+	}
 }
 
 static void
-panel_menu_bar_connect_menu_signals (PanelMenuBar  *menubar,
-				     GtkWidget    **menu)
+panel_menu_bar_append_applications_menu (GtkWidget    *menu,
+					 PanelMenuBar *menubar)
 {
-	/* intercept all right button clicks makes sure they don't
-	   go to the object itself */
-	g_signal_connect (*menu, "button_press_event",
-			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+	GtkWidget *item;
 
-	g_signal_connect (*menu, "destroy", G_CALLBACK (gtk_widget_destroyed),
-			  menu);
+	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-appending"))
+		return;
 
-	g_signal_connect_swapped (*menu, "show",
-				  G_CALLBACK (panel_menu_bar_show_menu),
-				  menubar);
+	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-appending", NULL);
 
-	g_signal_connect_swapped (*menu, "hide",
-				  G_CALLBACK (gtk_menu_shell_deselect), 
-				  menubar);
-}
+	item = panel_menu_items_create_action_item (PANEL_ACTION_RUN);
 
-static void
-panel_menu_bar_recreate_actions_menu (PanelMenuBar *menubar)
-{
-	if (menubar->priv->actions_menu) {
-		gtk_widget_destroy (menubar->priv->actions_menu);
-		menubar->priv->actions_menu = panel_menu_bar_create_actions_menu (menubar);
-		panel_menu_bar_connect_menu_signals (menubar,
-						     &menubar->priv->actions_menu);
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
-					   menubar->priv->actions_menu);
+	if (item != NULL) {
+		gboolean  add_separator;
+		GList    *children;
+
+		add_separator = FALSE;
+		children = gtk_container_get_children (GTK_CONTAINER (menu));
+
+		if (children != NULL) {
+			while (children->next != NULL)
+				children = children->next;
+			add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+		}
+
+		if (add_separator)
+			add_menu_separator (menu);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 }
 
 static GtkWidget *
-panel_menu_bar_create_actions_menu (PanelMenuBar *menubar)
+panel_menu_bar_create_applications_menu (PanelMenuBar *menubar)
 {
-	GtkWidget *actions_menu;
-	GtkWidget *item;
-	gboolean   separator_inserted;
-	gboolean   last_is_separator;
+	GtkWidget *applications_menu;
 
-	actions_menu = panel_create_menu ();
+	applications_menu = create_applications_menu ("applications.menu", NULL);
 
-	last_is_separator = FALSE;
+	g_signal_connect (applications_menu, "show",
+			  G_CALLBACK (panel_menu_bar_append_applications_menu),
+			  menubar);
 
-	item = menu_create_action_item (PANEL_ACTION_RUN);
-	if (item != NULL) {
-		gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu), item);
-		add_menu_separator (actions_menu);
-		last_is_separator = TRUE;
-	}
-
-	if (panel_is_program_in_path  ("gnome-search-tool")) {
-		item = menu_create_action_item (PANEL_ACTION_SEARCH);
-		if (item != NULL) {
-			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
-					       item);
-			last_is_separator = FALSE;
-		}
-	}
-
-	panel_recent_append_documents_menu (actions_menu);
-
-	if (panel_is_program_in_path ("gnome-screenshot")) {
-		item = menu_create_action_item (PANEL_ACTION_SCREENSHOT);
-		if (item != NULL) {
-			if (!last_is_separator)
-				add_menu_separator (actions_menu);
-			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
-					       item);
-			last_is_separator = FALSE;
-		}
-	}
-
-	separator_inserted = FALSE;
-
-	item = menu_create_action_item (PANEL_ACTION_LOCK);
-	if (item != NULL) {
-		add_menu_separator (actions_menu);
-		gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu), item);
-		separator_inserted = TRUE;
-	}
-
-	if (panel_is_program_in_path ("xscreensaver")) {
-		item = menu_create_action_item (PANEL_ACTION_LOGOUT);
-
-		if (item != NULL) {
-			if (!separator_inserted)
-				add_menu_separator (actions_menu);
-			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
-					       item);
-		}
-	}
-
-	return actions_menu;
+	return applications_menu;
 }
 
 static void
 panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 			      PanelMenuBarClass *klass)
 {
-	GtkWidget *item;
 	GtkWidget *image;
 
 	menubar->priv = PANEL_MENU_BAR_GET_PRIVATE (menubar);
 
 	menubar->priv->info = NULL;
 
-	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
-	panel_menu_bar_connect_menu_signals (menubar,
-					     &menubar->priv->applications_menu);
+	menubar->priv->applications_menu = panel_menu_bar_create_applications_menu (menubar);
 
-	menubar->priv->actions_menu = panel_menu_bar_create_actions_menu (menubar);
-	panel_menu_bar_connect_menu_signals (menubar,
-					     &menubar->priv->actions_menu);
+	menubar->priv->applications_item = gtk_image_menu_item_new_with_label (_("Applications"));
+	image = gtk_image_new_from_icon_name (PANEL_GNOME_LOGO_ICON,
+					      panel_menu_bar_icon_get_size ());
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
+				       image);
 
-	item = gtk_image_menu_item_new_with_label (_("Applications"));
-	image = gtk_image_new_from_stock (PANEL_STOCK_GNOME_LOGO,
-					  panel_menu_bar_icon_get_size ());
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->applications_item),
 				   menubar->priv->applications_menu);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menubar), item);
-
-	menubar->priv->actions_item = gtk_menu_item_new_with_label (_("Actions"));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
-				   menubar->priv->actions_menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
-			       menubar->priv->actions_item);
+			       menubar->priv->applications_item);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_actions_menu),
+	menubar->priv->places_item = panel_place_menu_item_new (FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
+			       menubar->priv->places_item);
+
+	menubar->priv->desktop_item = panel_desktop_menu_item_new (FALSE, TRUE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
+			       menubar->priv->desktop_item);
+
+	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_menus),
 				   menubar);
 }
 
@@ -208,12 +163,16 @@ panel_menu_bar_parent_set (GtkWidget *widget,
 
 	menubar->priv->panel = (PanelWidget *) widget->parent;
 
-	panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
-				       "menu_panel",
-				       menubar->priv->panel);
-	panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->actions_menu),
-				       "menu_panel",
-				       menubar->priv->panel);
+	if (menubar->priv->applications_menu)
+		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
+					       "menu_panel",
+					       menubar->priv->panel);
+	if (menubar->priv->places_item)
+		panel_place_menu_item_set_panel (menubar->priv->places_item,
+						 menubar->priv->panel);
+	if (menubar->priv->desktop_item)
+		panel_desktop_menu_item_set_panel (menubar->priv->desktop_item,
+						   menubar->priv->panel);
 }
 
 static void
@@ -238,13 +197,26 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 	panel_menu_bar_change_background (PANEL_MENU_BAR (widget));
 }
 
+static void
+panel_menu_bar_finalize (GObject *object)
+{
+	PanelMenuBar *menubar = (PanelMenuBar *) object;
+
+	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_bar_recreate_menus),
+				      menubar);
+
+	parent_class->finalize (object);
+}
+
 static void
 panel_menu_bar_class_init (PanelMenuBarClass *klass)
 {
-	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+	GObjectClass   *gobject_class = (GObjectClass   *) klass;
+	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	gobject_class->finalize  = panel_menu_bar_finalize;
 	widget_class->parent_set = panel_menu_bar_parent_set;
 	widget_class->size_allocate = panel_menu_bar_size_allocate;
 
@@ -285,6 +257,24 @@ panel_menu_bar_get_type (void)
 	return type;
 }
 
+static gboolean
+panel_menu_bar_on_expose (GtkWidget      *widget,
+			  GdkEventExpose *event,
+			  gpointer        data)
+{
+	PanelMenuBar *menubar = data;
+
+	if (GTK_WIDGET_HAS_FOCUS (menubar))
+		gtk_paint_focus (widget->style,
+				 widget->window, 
+				 GTK_WIDGET_STATE (menubar),
+				 NULL,
+				 widget,
+				 "menubar-applet",
+				 0, 0, -1, -1);
+	return FALSE;
+}
+
 static void
 panel_menu_bar_load (PanelWidget *panel,
 		     gboolean     locked,
@@ -312,8 +302,14 @@ panel_menu_bar_load (PanelWidget *panel,
 				   GTK_STOCK_HELP,
 				   _("_Help"),
 				   NULL);
-
+	g_signal_connect_after (G_OBJECT (menubar), "focus-in-event",
+				G_CALLBACK (gtk_widget_queue_draw), menubar);
+	g_signal_connect_after (G_OBJECT (menubar), "focus-out-event",
+				G_CALLBACK (gtk_widget_queue_draw), menubar);
+	g_signal_connect_after (G_OBJECT (menubar), "expose-event",
+				G_CALLBACK (panel_menu_bar_on_expose), menubar);
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
+	GTK_WIDGET_SET_FLAGS (menubar, GTK_CAN_FOCUS);
 }
 
 void
@@ -369,7 +365,7 @@ panel_menu_bar_popup_menu (PanelMenuBar *menubar,
 	 * We need to call _gtk_menu_shell_activate() here as is done in 
 	 * window_key_press_handler in gtkmenubar.c which pops up menu
 	 * when F10 is pressed.
-         *
+	 *
 	 * As that function is private its code is replicated here.
 	 */
 	menu_shell = GTK_MENU_SHELL (menubar);
@@ -395,7 +391,7 @@ void panel_menu_bar_change_background (PanelMenuBar *menubar)
 	gtk_widget_set_style (GTK_WIDGET (menubar), NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (GTK_WIDGET (menubar), rc_style);
-	g_object_unref (rc_style);
+	gtk_rc_style_unref (rc_style);
 
 	switch (panel_background_get_type (&menubar->priv->panel->background)) {
 	case PANEL_BACK_NONE:
@@ -437,6 +433,7 @@ void panel_menu_bar_change_background (PanelMenuBar *menubar)
 			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
 		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
 		gtk_widget_set_style (GTK_WIDGET (menubar), style);
+		g_object_unref (style);
 
 		g_object_unref (pixmap);
 		break;
